;
;  This Program is the equivalent to "cFlash.c" but written in 
;   assembly language, taking advantage of ALL the features of 
;   PIC MCU assembly language - bit instructions, bit skip 
;   instructions and decrement and skip instructions.  
;
;  Hardware Notes:
;   PIC16F684 running at 4 MHz Using the Internal Clock
;   Internal Reset is Used
;   RA4 - LED Positive
;   RA5 - LED Negative
;
;
;  Myke Predko
;  04.11.17
;
  LIST R=DEC
 INCLUDE "p16f684.inc"

 __CONFIG _FCMEN_OFF & _IESO_OFF & _BOD_OFF & _CPD_OFF & _CP_OFF & _MCLRE_ON & _PWRTE_ON & _WDT_OFF & _INTOSCIO

#DEFINE VBB_SIM 1	;The original code has been modified for better simulation speed
					;set VBB_SIM 0 when executing on real-hardware 

;  Variables
 CBLOCK 0x20
Dlay:2
 ENDC

  PAGE
;  Mainline

 org     0

  nop                           ;  For ICD Debug

  clrf    PORTA                 ;  Initialize I/O Bits to Off
  movlw   7                     ;  Turn off Comparators
  movwf   CMCON0
  bsf     STATUS, RP0           ;  Execute out of Bank 1
  clrf    ANSEL ^ 0x080         ;  All Bits are Digital
  bcf     TRISA ^ 0x080, 4      ;  Replaces:    movlw   b'001111'
  bcf     TRISA ^ 0x080, 5      ;               movwf   TRISA ^ 0x080
  bcf     STATUS, RP0           ;  Return Execution to Bank 0

Loop:                           ;  Return Here after D0 Toggle
  clrf    Dlay + 1              ;  High 8 Bits for Delay
  clrf    Dlay                  ;  Low 8 Bits for Delay

#if VBB_SIM 
	MOVLW .5					;  VBB doesn't simulate as fast as real hardware so
	MOVWF Dlay + 1				;  sometimes small changes in delay loops are used 
								;  to make the simulated circuit behave more closley
								;  to what it looks like on the real-circuit.
								;  In this example we reduce the loop delay count
#endif

DlayLoop:
  goto    $ + 1                 ;  Three Cycle Delay
  nop
  movlw   1                     ;  Decrement the Inside Loop
  subwf   Dlay, f
  btfss   STATUS, Z             ;  Skip if Zero Flag is Set
   goto   DlayLoop              ;  Else, Loop Again
                                ;  Replaces:    movf    STATUS, w
                                ;               andlw   1 << 2
                                ;               addwf   PCL, f
                                ;               goto    DlayLoop
                                ;               nop
                                ;               nop
                                ;               nop
  decf    Dlay + 1, f           ;  Decrement the High Byte Until 
  btfss   STATUS, Z             ;   It Equals Zero
   goto   DlayLoop              ;  Replaces:    movlw   1
                                ;               subwf   Dlay + 1, f
                                ;               movf    STATUS, w
                                ;               andlw   1 << 2
                                ;               addwf   PCL, f
                                ;               goto    DlayLoop
                                ;               nop
                                ;               nop
                                ;               nop

  btfsc   PORTA, 4              ;  Toggle RA4
   goto   RA4Set
RA4Reset:                       ;  RA4 is Off, Turn it On
  bsf     PORTA, 4
  goto    Loop
RA4Set:                         ;  RA4 is On, Turn it Off
  bcf     PORTA, 4  
  goto    Loop                  ;  Repeat


  end                   
